@@ -27,7 +27,11 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-"""Flagvalues module - Registry of 'Flag' objects."""
+"""Flagvalues module - Registry of 'Flag' objects.
+
+Instead of importing this module directly, it's preferable to import the
+flags package and use the aliases defined at the package level.
+"""
 
 import hashlib
 import logging
@@ -36,11 +40,13 @@ import struct
 import sys
 import traceback
 import warnings
+from xml.dom import minidom
+
+import six
 
 from gflags import _helpers
 from gflags import exceptions
 from gflags import flag as _flag
-from gflags import validators as gflags_validators
 
 # Add flagvalues module to disclaimed module ids.
 _helpers.disclaim_module_ids.add(id(sys.modules[__name__]))
@@ -57,6 +63,13 @@ _UNPARSED_FLAG_ACCESS_ENV_NAME = 'GFLAGS_ALLOW_UNPARSED_FLAG_ACCESS'
 # default.
 _UNPARSED_ACCESS_DISABLED_PERCENT = 0
 
+# b/32278439 will change flag parsing to use GNU-style scanning by default.
+# This environment variable allows users to force setting the default parsing
+# style. Do NOT rely on it. It will be removed as part of b/32278439.
+_USE_GNU_GET_OPT_ENV_NAME = 'GFLAGS_USE_GNU_GET_OPT'
+
+
+
 
 class FlagValues(object):
   """Registry of 'Flag' objects.
@@ -118,9 +131,15 @@ class FlagValues(object):
     # Bool: True if Reset() was called.
     self.__dict__['__reset_called'] = False
 
-    # Set if we should use new style gnu_getopt rather than getopt when parsing
-    # the args.  Only possible with Python 2.3+
-    self.UseGnuGetOpt(False)
+    # None or Method(name, value) to call from __setattr__ for an unknown flag.
+    self.__dict__['__set_unknown'] = None
+
+    if _USE_GNU_GET_OPT_ENV_NAME in os.environ:
+      self.__dict__['__use_gnu_getopt'] = (
+          os.environ[_USE_GNU_GET_OPT_ENV_NAME] == '1')
+    else:
+      # By default don't use the GNU-style scanning when parsing the args.
+      self.__dict__['__use_gnu_getopt'] = False
 
   def UseGnuGetOpt(self, use_gnu_getopt=True):
     """Use GNU-style scanning. Allows mixing of flag and non-flag arguments.
@@ -202,6 +221,47 @@ class FlagValues(object):
     if flag not in key_flags:
       key_flags.append(flag)
 
+  def _FlagIsRegistered(self, flag_obj):
+    """Checks whether a Flag object is registered under long name or short name.
+
+    Args:
+      flag_obj: A Flag object.
+
+    Returns:
+      A boolean: True iff flag_obj is registered under long name or short name.
+    """
+    flag_dict = self.FlagDict()
+    # Check whether flag_obj is registered under its long name.
+    name = flag_obj.name
+    if flag_dict.get(name, None) == flag_obj:
+      return True
+    # Check whether flag_obj is registered under its short name.
+    short_name = flag_obj.short_name
+    if (short_name is not None and
+        flag_dict.get(short_name, None) == flag_obj):
+      return True
+    return False
+
+  def _CleanupUnregisteredFlagFromModuleDicts(self, flag_obj):
+    """Cleanup unregistered flags from all module -> [flags] dictionaries.
+
+    If flag_obj is registered under either its long name or short name, it
+    won't be removed from the dictionaries.
+
+    Args:
+      flag_obj: A flag object.
+    """
+    if self._FlagIsRegistered(flag_obj):
+      return
+    for flags_by_module_dict in (self.FlagsByModuleDict(),
+                                 self.FlagsByModuleIdDict(),
+                                 self.KeyFlagsByModuleDict()):
+      for flags_in_module in six.itervalues(flags_by_module_dict):
+        # While (as opposed to if) takes care of multiple occurrences of a
+        # flag in the list for the same module.
+        while flag_obj in flags_in_module:
+          flags_in_module.remove(flag_obj)
+
   def _GetFlagsDefinedByModule(self, module):
     """Returns the list of flags defined by a module.
 
@@ -256,9 +316,16 @@ class FlagValues(object):
       If no such module exists (i.e. no flag with this name exists),
       we return default.
     """
-    for module, flags in self.FlagsByModuleDict().iteritems():
+    registered_flag = self.FlagDict().get(flagname)
+    if registered_flag is None:
+      return default
+    for module, flags in six.iteritems(self.FlagsByModuleDict()):
       for flag in flags:
-        if flag.name == flagname or flag.short_name == flagname:
+        # It must compare the flag with the one in FlagDict. This is because a
+        # flag might be overridden only for its long name (or short name),
+        # and only its short name (or long name) is considered registered.
+        if (flag.name == registered_flag.name and
+            flag.short_name == registered_flag.short_name):
           return module
     return default
 
@@ -275,19 +342,61 @@ class FlagValues(object):
       If no such module exists (i.e. no flag with this name exists),
       we return default.
     """
-    for module_id, flags in self.FlagsByModuleIdDict().iteritems():
+    registered_flag = self.FlagDict().get(flagname)
+    if registered_flag is None:
+      return default
+    for module_id, flags in six.iteritems(self.FlagsByModuleIdDict()):
       for flag in flags:
-        if flag.name == flagname or flag.short_name == flagname:
+        # It must compare the flag with the one in FlagDict. This is because a
+        # flag might be overridden only for its long name (or short name),
+        # and only its short name (or long name) is considered registered.
+        if (flag.name == registered_flag.name and
+            flag.short_name == registered_flag.short_name):
           return module_id
     return default
 
+  def _RegisterUnknownFlagSetter(self, setter):
+    """Allow set default values for undefined flags.
+
+    Args:
+      setter: Method(name, value) to call to __setattr__ an unknown flag.
+        Must raise NameError or ValueError for invalid name/value.
+    """
+    self.__dict__['__set_unknown'] = setter
+
+  def _SetUnknownFlag(self, name, value):
+    """Returns value if setting flag |name| to |value| returned True.
+
+    Args:
+      name: Name of the flag to set.
+      value: Value to set.
+
+    Returns:
+      Flag value on successful call.
+
+    Raises:
+      UnrecognizedFlagError
+      IllegalFlagValueError
+    """
+    setter = self.__dict__['__set_unknown']
+    if setter:
+      try:
+        setter(name, value)
+        return value
+      except (TypeError, ValueError):  # Flag value is not valid.
+        raise exceptions.IllegalFlagValueError('"{1}" is not valid for --{0}'
+                                               .format(name, value))
+      except NameError:  # Flag name is not valid.
+        pass
+    raise exceptions.UnrecognizedFlagError(name, value)
+
   def AppendFlagValues(self, flag_values):
     """Appends flags registered in another FlagValues instance.
 
     Args:
       flag_values: registry to copy from
     """
-    for flag_name, flag in flag_values.FlagDict().iteritems():
+    for flag_name, flag in six.iteritems(flag_values.FlagDict()):
       # Each flags with shortname appears here twice (once under its
       # normal name, and again with its short name).  To prevent
       # problems (DuplicateFlagError) with double flag registration, we
@@ -297,7 +406,7 @@ class FlagValues(object):
         try:
           self[flag_name] = flag
         except exceptions.DuplicateFlagError:
-          raise exceptions.DuplicateFlagError(
+          raise exceptions.DuplicateFlagError.from_flag(
               flag_name, self, other_flag_values=flag_values)
 
   def RemoveFlagValues(self, flag_values):
@@ -313,15 +422,15 @@ class FlagValues(object):
     """Registers a new flag variable."""
     fl = self.FlagDict()
     if not isinstance(flag, _flag.Flag):
-      raise exceptions.IllegalFlagValue(flag)
+      raise exceptions.IllegalFlagValueError(flag)
     if str is bytes and isinstance(name, unicode):
       # When using Python 2 with unicode_literals, allow it but encode it
       # into the bytes type we require.
       name = name.encode('utf-8')
     if not isinstance(name, type('')):
-      raise exceptions.FlagsError('Flag name must be a string')
+      raise exceptions.Error('Flag name must be a string')
     if not name:
-      raise exceptions.FlagsError('Flag name cannot be empty')
+      raise exceptions.Error('Flag name cannot be empty')
     if name in fl and not flag.allow_override and not fl[name].allow_override:
       module, module_name = _helpers.GetCallingModuleObjectAndName()
       if (self.FindModuleDefiningFlag(name) == module_name and
@@ -330,17 +439,26 @@ class FlagValues(object):
         # but a different ID, we can stop here because it indicates that the
         # module is simply being imported a subsequent time.
         return
-      raise exceptions.DuplicateFlagError(name, self)
+      raise exceptions.DuplicateFlagError.from_flag(name, self)
     short_name = flag.short_name
+    # If a new flag overrides an old one, we need to cleanup the old flag's
+    # modules if it's not registered.
+    flags_to_cleanup = set()
     if short_name is not None:
       if (short_name in fl and not flag.allow_override and
           not fl[short_name].allow_override):
-        raise exceptions.DuplicateFlagError(short_name, self)
+        raise exceptions.DuplicateFlagError.from_flag(short_name, self)
+      if short_name in fl and fl[short_name] != flag:
+        flags_to_cleanup.add(fl[short_name])
       fl[short_name] = flag
     if (name not in fl  # new flag
         or fl[name].using_default_value
         or not flag.using_default_value):
+      if name in fl and fl[name] != flag:
+        flags_to_cleanup.add(fl[name])
       fl[name] = flag
+    for f in flags_to_cleanup:
+      self._CleanupUnregisteredFlagFromModuleDicts(f)
 
   def __dir__(self):
     """Returns list of names of all defined flags.
@@ -411,13 +529,12 @@ class FlagValues(object):
             error_message + ' This will raise an exception in the future.',
             RuntimeWarning,
             stacklevel=2)
-        traceback.print_stack()
         # Force logging.exception() to behave realistically, but don't propagate
         # exception up. Allow flag value to be returned (for now).
         try:
           raise exceptions.UnparsedFlagAccessError(error_message)
         except exceptions.UnparsedFlagAccessError:
-          logging.exception(error_message)
+            logging.exception(error_message)
         return fl[name].value
       else:
         raise exceptions.UnparsedFlagAccessError(error_message)
@@ -427,18 +544,16 @@ class FlagValues(object):
     fl = self.FlagDict()
     if name in self.__dict__['__hiddenflags']:
       raise AttributeError(name)
-    if fl[name].Type() == '[C++]':
-      fl[name].parser.Parse(value)
-      fl[name].value = value
-    else:
-      fl[name].value = value
-      self._AssertValidators(fl[name].validators)
+    if name not in fl:
+      return self._SetUnknownFlag(name, value)
+    fl[name].value = value
+    self._AssertValidators(fl[name].validators)
     fl[name].using_default_value = False
     return value
 
   def _AssertAllValidators(self):
     all_validators = set()
-    for flag in self.FlagDict().itervalues():
+    for flag in six.itervalues(self.FlagDict()):
       for validator in flag.validators:
         all_validators.add(validator)
     self._AssertValidators(all_validators)
@@ -448,61 +563,35 @@ class FlagValues(object):
 
     Asserts validators in the order they were created.
     Args:
-      validators: Iterable(gflags_validators.Validator), validators to be
+      validators: Iterable(validators.Validator), validators to be
         verified
     Raises:
       AttributeError: if validators work with a non-existing flag.
-      IllegalFlagValue: if validation fails for at least one validator
+      IllegalFlagValueError: if validation fails for at least one validator
     """
     for validator in sorted(
         validators, key=lambda validator: validator.insertion_index):
       try:
-        validator.Verify(self)
-      except gflags_validators.Error as e:
-        message = validator.PrintFlagsWithValues(self)
-        raise exceptions.IllegalFlagValue('%s: %s' % (message, str(e)))
-
-  def _FlagIsRegistered(self, flag_obj):
-    """Checks whether a Flag object is registered under some name.
-
-    Note: this is non trivial: in addition to its normal name, a flag
-    may have a short name too.  In self.FlagDict(), both the normal and
-    the short name are mapped to the same flag object.  E.g., calling
-    only "del FLAGS.short_name" is not unregistering the corresponding
-    Flag object (it is still registered under the longer name).
-
-    Args:
-      flag_obj: A Flag object.
-
-    Returns:
-      A boolean: True iff flag_obj is registered under some name.
-    """
-    flag_dict = self.FlagDict()
-    # Check whether flag_obj is registered under its long name.
-    name = flag_obj.name
-    if flag_dict.get(name, None) == flag_obj:
-      return True
-    # Check whether flag_obj is registered under its short name.
-    short_name = flag_obj.short_name
-    if (short_name is not None and
-        flag_dict.get(short_name, None) == flag_obj):
-      return True
-    # The flag cannot be registered under any other name, so we do not
-    # need to do a full search through the values of self.FlagDict().
-    return False
+        validator.verify(self)
+      except exceptions.ValidationError as e:
+        message = validator.print_flags_with_values(self)
+        raise exceptions.IllegalFlagValueError('%s: %s' % (message, str(e)))
 
   def __delattr__(self, flag_name):
     """Deletes a previously-defined flag from a flag object.
 
     This method makes sure we can delete a flag by using
 
-      del flag_values_object.<flag_name>
+      del FLAGS.<flag_name>
 
     E.g.,
 
       gflags.DEFINE_integer('foo', 1, 'Integer flag.')
       del gflags.FLAGS.foo
 
+    If a flag is also registered by its the other name (long name or short
+    name), the other name won't be deleted.
+
     Args:
       flag_name: A string, the name of the flag to be deleted.
 
@@ -516,28 +605,30 @@ class FlagValues(object):
     flag_obj = fl[flag_name]
     del fl[flag_name]
 
-    if not self._FlagIsRegistered(flag_obj):
-      # If the Flag object indicated by flag_name is no longer
-      # registered (please see the docstring of _FlagIsRegistered), then
-      # we delete the occurrences of the flag object in all our internal
-      # dictionaries.
-      self.__RemoveFlagFromDictByModule(self.FlagsByModuleDict(), flag_obj)
-      self.__RemoveFlagFromDictByModule(self.FlagsByModuleIdDict(), flag_obj)
-      self.__RemoveFlagFromDictByModule(self.KeyFlagsByModuleDict(), flag_obj)
+    self._CleanupUnregisteredFlagFromModuleDicts(flag_obj)
 
-  def __RemoveFlagFromDictByModule(self, flags_by_module_dict, flag_obj):
-    """Removes a flag object from a module -> list of flags dictionary.
+  def _RemoveAllFlagAppearances(self, name):
+    """Removes flag with name for all appearances.
+
+    A flag can be registered with its long name and an optional short name.
+    This method removes both of them. This is different than __delattr__.
 
     Args:
-      flags_by_module_dict: A dictionary that maps module names to lists of
-        flags.
-      flag_obj: A flag object.
+      name: Either flag's long name or short name.
+
+    Raises:
+      UnrecognizedFlagError: When flag name is not found.
     """
-    for unused_module, flags_in_module in flags_by_module_dict.iteritems():
-      # while (as opposed to if) takes care of multiple occurrences of a
-      # flag in the list for the same module.
-      while flag_obj in flags_in_module:
-        flags_in_module.remove(flag_obj)
+    flag_dict = self.FlagDict()
+    if name not in flag_dict:
+      raise exceptions.UnrecognizedFlagError(name)
+    flag = flag_dict[name]
+    names_to_remove = {name}
+    names_to_remove.add(flag.name)
+    if flag.short_name:
+      names_to_remove.add(flag.short_name)
+    for n in names_to_remove:
+      self.__delattr__(n)
 
   def SetDefault(self, name, value):
     """Changes the default value (and current value) of the named flag object.
@@ -550,18 +641,20 @@ class FlagValues(object):
       value: The new default value.
 
     Raises:
-      AttributeError: When there is no registered flag named name.
+      UnrecognizedFlagError: When there is no registered flag named name.
+      IllegalFlagValueError: When value is not valid.
     """
     fl = self.FlagDict()
     if name not in fl:
-      raise AttributeError(name)
+      self._SetUnknownFlag(name, value)
+      return
     if self.IsParsed():
       logging.warn(
           'FLAGS.SetDefault called on flag "%s" after flag parsing. Call this '
           'method at the top level of a module to avoid overwriting the value '
           'passed at the command line.',
           name)
-    fl[name].SetDefault(value)
+    fl[name]._set_default(value)  # pylint: disable=protected-access
     self._AssertValidators(fl[name].validators)
 
   def __contains__(self, name):
@@ -586,7 +679,7 @@ class FlagValues(object):
        The list of arguments not parsed as options, including argv[0].
 
     Raises:
-       FlagsError: on any parsing error.
+       Error: on any parsing error.
        ValueError: on flag value parsing error.
     """
     if not argv:
@@ -623,7 +716,7 @@ class FlagValues(object):
 
     This function goes through args and does the bulk of the flag parsing.
     It will find the corresponding flag in our flag dictionary, and call its
-    .Parse() method on the flag value.
+    .parse() method on the flag value.
 
     Args:
       args: List of strings with the arguments to parse.
@@ -636,7 +729,7 @@ class FlagValues(object):
         undefok: Set of flags that were given via --undefok.
 
     Raises:
-       FlagsError: on any parsing error.
+       Error: on any parsing error.
        ValueError: on flag value parsing error.
     """
     unknown_flags, unparsed_args, undefok = [], [], set()
@@ -651,7 +744,7 @@ class FlagValues(object):
         try:
           return next(args) if value is None else value
         except StopIteration:
-          raise exceptions.FlagsError('Missing value for flag ' + arg)
+          raise exceptions.Error('Missing value for flag ' + arg)
 
       if not arg.startswith('-'):
         # A non-argument: default is break, GNU is skip.
@@ -681,6 +774,8 @@ class FlagValues(object):
 
       # --undefok is a special case.
       if name == 'undefok':
+        if known_only:
+          unparsed_args.append(arg)
         value = GetValue()
         undefok.update(v.strip() for v in value.split(','))
         undefok.update('no' + v.strip() for v in value.split(','))
@@ -700,7 +795,7 @@ class FlagValues(object):
 
 
       if flag:
-        flag.Parse(value)
+        flag.parse(value)
         flag.using_default_value = False
       elif known_only:
         unparsed_args.append(arg)
@@ -726,9 +821,11 @@ class FlagValues(object):
   def Reset(self):
     """Resets the values to the point before FLAGS(argv) was called."""
     for f in self.FlagDict().values():
-      f.Unparse()
+      f.unparse()
+    # We log this message before marking flags as unparsed to avoid a
+    # problem when the logging library causes flags access.
+    logging.info('Reset() called; flags access will now raise errors.')
     self.__dict__['__flags_parsed'] = False
-    logging.info('Reset() called; flags access will raise errors now.')
     self.__dict__['__reset_called'] = True
 
   def RegisteredFlags(self):
@@ -768,7 +865,7 @@ class FlagValues(object):
       modules = sorted(flags_by_module)
 
       # Print the help for the main module first, if possible.
-      main_module = _helpers.GetMainModule()
+      main_module = sys.argv[0]
       if main_module in modules:
         modules.remove(main_module)
         modules = [main_module] + modules
@@ -833,7 +930,7 @@ class FlagValues(object):
     Returns:
       string describing the key flags of a module.
     """
-    return self.ModuleHelp(_helpers.GetMainModule())
+    return self.ModuleHelp(sys.argv[0])
 
   def __RenderFlagList(self, flaglist, output_lines, prefix='  '):
     fl = self.FlagDict()
@@ -872,7 +969,7 @@ class FlagValues(object):
             '(%s)' % flag.parser.syntactic_help, indent=prefix+'  ')
       output_lines.append(flaghelp)
 
-  def get(self, name, default):  # pylint: disable=invalid-name
+  def get_flag_value(self, name, default):  # pylint: disable=invalid-name
     """Returns the value of a flag (if not None) or a default value.
 
     Args:
@@ -889,6 +986,9 @@ class FlagValues(object):
     else:
       return default
 
+  # TODO(b/32098517): Remove this.
+  get = get_flag_value
+
   def __IsFlagFileDirective(self, flag_string):
     """Checks whether flag_string contain a --flagfile=<foo> directive."""
     if isinstance(flag_string, type('')):
@@ -918,14 +1018,14 @@ class FlagValues(object):
       str filename from a flagfile_str of form -[-]flagfile=filename.
 
     Raises:
-      FlagsError: when illegal --flagfile provided.
+      Error: when illegal --flagfile provided.
     """
     if flagfile_str.startswith('--flagfile='):
       return os.path.expanduser((flagfile_str[(len('--flagfile=')):]).strip())
     elif flagfile_str.startswith('-flagfile='):
       return os.path.expanduser((flagfile_str[(len('-flagfile=')):]).strip())
     else:
-      raise exceptions.FlagsError(
+      raise exceptions.Error(
           'Hit illegal --flagfile type: %s' % flagfile_str)
 
   def __GetFlagFileLines(self, filename, parsed_file_stack=None):
@@ -1008,7 +1108,7 @@ class FlagValues(object):
       from any flagfile(s).
 
     Raises:
-      IllegalFlagValue: when --flagfile provided with no argument.
+      IllegalFlagValueError: when --flagfile provided with no argument.
 
     References: Global gflags.FLAG class instance.
 
@@ -1041,7 +1141,8 @@ class FlagValues(object):
         # next arg really is part of this one.
         if current_arg == '--flagfile' or current_arg == '-flagfile':
           if not rest_of_args:
-            raise exceptions.IllegalFlagValue('--flagfile with no argument')
+            raise exceptions.IllegalFlagValueError(
+                '--flagfile with no argument')
           flag_filename = os.path.expanduser(rest_of_args[0])
           rest_of_args = rest_of_args[1:]
         else:
@@ -1089,7 +1190,7 @@ class FlagValues(object):
     s = ''
     for flag in self.FlagDict().values():
       if flag.value is not None:
-        s += flag.Serialize() + '\n'
+        s += flag.serialize() + '\n'
     return s
 
   def AppendFlagsIntoFile(self, filename):
@@ -1119,23 +1220,22 @@ class FlagValues(object):
     Args:
       outfile: File object we write to.  Default None means sys.stdout.
     """
-    outfile = outfile or sys.stdout
+    doc = minidom.Document()
+    all_flag = doc.createElement('AllFlags')
+    doc.appendChild(all_flag)
 
-    outfile.write('<?xml version=\"1.0\"?>\n')
-    outfile.write('<AllFlags>\n')
-    indent = '  '
-    _helpers.WriteSimpleXMLElement(outfile, 'program',
-                                   os.path.basename(sys.argv[0]), indent)
+    all_flag.appendChild(_helpers.CreateXMLDOMElement(
+        doc, 'program', os.path.basename(sys.argv[0])))
 
     usage_doc = sys.modules['__main__'].__doc__
     if not usage_doc:
       usage_doc = '\nUSAGE: %s [flags]\n' % sys.argv[0]
     else:
       usage_doc = usage_doc.replace('%s', sys.argv[0])
-    _helpers.WriteSimpleXMLElement(outfile, 'usage', usage_doc, indent)
+    all_flag.appendChild(_helpers.CreateXMLDOMElement(doc, 'usage', usage_doc))
 
     # Get list of key flags for the main module.
-    key_flags = self._GetKeyFlagsForModule(_helpers.GetMainModule())
+    key_flags = self._GetKeyFlagsForModule(sys.argv[0])
 
     # Sort flags by declaring module name and next by flag name.
     flags_by_module = self.FlagsByModuleDict()
@@ -1146,21 +1246,16 @@ class FlagValues(object):
       flag_list.sort()
       for unused_flag_name, flag in flag_list:
         is_key = flag in key_flags
-        flag.WriteInfoInXMLFormat(outfile, module_name,
-                                  is_key=is_key, indent=indent)
+        all_flag.appendChild(flag._create_xml_dom_element(  # pylint: disable=protected-access
+            doc, module_name, is_key=is_key))
 
-    outfile.write('</AllFlags>\n')
+    outfile = outfile or sys.stdout
+    if six.PY2:
+      outfile.write(doc.toprettyxml(indent='  ', encoding='utf-8'))
+    else:
+      outfile.write(
+          doc.toprettyxml(indent='  ', encoding='utf-8').decode('utf-8'))
     outfile.flush()
 
-  def AddValidator(self, validator):
-    """Register new flags validator to be checked.
-
-    Args:
-      validator: gflags_validators.Validator
-    Raises:
-      AttributeError: if validators work with a non-existing flag.
-    """
-    for flag_name in validator.GetFlagsNames():
-      self.GetFlag(flag_name).validators.append(validator)
 
 _helpers.SPECIAL_FLAGS = FlagValues()
